@@ -34,7 +34,18 @@ classdef arduino < handle
   ## for additional properties to pass to the creation of the 
   ## arduino object.
   ##
-  ## Currently properties are ignored.
+  ## Currently properties are ignored, with the exception of:
+  ## @table @asis
+  ## @item debug
+  ## true / false flag for whether setting debug (default false)
+  ## @item forcebuildon
+  ## true / false flag for whether to force show of the arduino IDE to
+  ## rebuild the installed code on the arduino (default false)
+  ## @item baudrate (read only)
+  ## the communications baudrate to the board. (default 9600)
+  ## @item libraries
+  ## The libraries to be enabled on the arduino board. (default uses whatever is already installed)
+  ## @end table
   ##
   ## if the arduino function is called without parameters, it will scan
   ## for the first available arduino it can find and connect to it.
@@ -48,12 +59,14 @@ classdef arduino < handle
   ## @item name
   ## name assigned to the arduino object
   ## @item debug
-  ## true / false flag for whether debug in turned on
+  ## true / false flag for whether debug is turned on
   ## @item forcebuildon
   ## true / false flag for whether to force show of the arduino IDE to
   ## reprogram the arduino
   ## @item port (read only)
   ## the communications port the board is connected to.
+  ## @item baudrate (read only)
+  ## the communications baudrate to the board.
   ## @item board  (read only)
   ## The name of the board type that the arduino connected to
   ## @item libraries (read only)
@@ -84,6 +97,7 @@ classdef arduino < handle
     AnalogReference = 5.0;
     Board = "";
     Port = "";
+    BaudRate = 9600;
   endproperties
 
   properties (SetAccess = private, Hidden = true)
@@ -101,7 +115,9 @@ classdef arduino < handle
         endif
 
         this.name = "arduino";
-        this = __initArduino__ (this, arduinos{1}.port, arduinos{1}.board);
+        c = arduinoio.getBoardConfig(arduinos{1}.board);
+        this.BaudRate = c.baudrate;
+        this = __initArduino__ (this, arduinos{1}.port, arduinos{1}.board, 0);
       elseif (nargin == 1)
         arg0 = varargin{1};
         if (isa (arg0, "arduino"))
@@ -114,7 +130,7 @@ classdef arduino < handle
           # port given
           this.name = "arduino";
           this.connected = false;
-          this = __initArduino__ (this, arg0, "");
+          this = __initArduino__ (this, arg0, "", 0);
         else
           error ("arduino: port must be a string");         
         endif
@@ -129,6 +145,8 @@ classdef arduino < handle
           endif
           port = arduinos{1}.port;
           board = arduinos{1}.board;
+          c = arduinoio.getBoardConfig(arduinos{1}.board);
+          this.BaudRate = c.baudrate;
         elseif !ischar (port)
           error ("arduino: port must be a string");         
         endif
@@ -148,6 +166,8 @@ classdef arduino < handle
         requiredlibs = {};
         forcebuild = false;
         forcebuildon = false;
+        scan_only = false;
+
         for i = 3:2:nargin
           propname = tolower (varargin{i});
           propvalue = varargin{i+1};
@@ -158,6 +178,10 @@ classdef arduino < handle
               this.debug = 1;
             endif
           endif
+          if strcmp (propname,"_scan_only")
+            # internal property
+            scan_only = propvalue;
+          endif
           if strcmp (propname,"libraries")
             if ischar (propvalue)
               requiredlibs{end+1} = propvalue;
@@ -174,6 +198,16 @@ classdef arduino < handle
               error ("arduino: expect forcebuildon to be true or false");
             endif
           endif
+          if strcmp (propname,"baudrate")
+            if !isnumeric(propvalue)
+              error ("arduino: expect baudrate to be numeric");
+            else
+              this.BaudRate = int32(propvalue);
+              if this.BaudRate < 1200
+                 error ("arduino: Invalid baudrate");
+              endif
+            endif
+          endif
           # older option that probally should remove
           if strcmp (propname,"forcebuild")
             if islogical (propvalue) || (isnumeric(propvalue) && (propvalue == 1 || propvalue == 0))
@@ -184,14 +218,14 @@ classdef arduino < handle
           endif
         endfor
 
-        this = __initArduino__ (this, port, board);
+        this = __initArduino__ (this, port, board, scan_only);
 
         # check have requested libs
         reprogram = false;
 
         if forcebuildon
           reprogram = true;
-        else
+        elseif ! scan_only
           availablelibs = listArduinoLibraries ();
 
           for i = 1:numel (requiredlibs)
@@ -217,11 +251,11 @@ classdef arduino < handle
           # free arduino resources, reprom and then reinit
           this = __freeArduino__(this);
 
-          if !arduinosetup ('libraries', requiredlibs);
+          if !arduinosetup ('libraries', requiredlibs, 'baudrate', this.BaudRate)
             error ("arduinosetup returned a failure, so did not reprogram")
           endif
 
-          this = __initArduino__ (this, port, board);
+          this = __initArduino__ (this, port, board, 0);
         endif
       endif
     endfunction
