@@ -22,7 +22,7 @@ function out = __getBytesAvailable__ (obj)
 
   ar = obj.parent;
 
-  [tmp, sz] = sendCommand (ar, "serial", ARDUINO_SERIAL_STAT, [obj.device.id]);
+  [tmp, sz] = sendCommand (ar, "serial", ARDUINO_SERIAL_STAT, [obj.devinfo.id]);
   out = uint16(tmp(2));
   
 endfunction
