@@ -37,7 +37,7 @@ function out = readRegister(dev, reg, numbytes, precision)
   if isempty(endian)
     [~, ~, endian] = computer ();
   endif
-  persistent ARDUINO_I2C_READREG = 5;
+  persistent ARDUINO_I2C_READREG = 9;
 
   if nargin < 3 || nargin > 4
     print_usage ();
@@ -68,14 +68,14 @@ function out = readRegister(dev, reg, numbytes, precision)
 
   if (strcmp (precision,'uint16'))
     reg = uint16(reg);
-    if (endian == 'L')
+    if (endian != getEndian(dev.parent))
       reg = swapbytes (reg); 
     endif
     reg = typecast (reg, 'uint8');
     regsz = 2;
   elseif (strcmp (precision,'int16'))
     reg = int16(reg);
-    if (endian == 'L')
+    if (endian != getEndian(dev.parent))
       reg = swabytes (reg); 
     endif
     reg = typecast (reg, 'uint8');
@@ -89,7 +89,7 @@ function out = readRegister(dev, reg, numbytes, precision)
     regsz = 1;
   endif
   % read  reg
-  [tmp, sz] = sendCommand (dev.parent, "i2c", ARDUINO_I2C_READREG, [dev.devinfo.address regsz reg numbytes*regsz]);
+  [tmp, sz] = sendCommand (dev.parent, "i2c", ARDUINO_I2C_READREG, [dev.devinfo.bus dev.devinfo.address regsz reg numbytes*regsz]);
   # skip address and and regsz and return the data
   out = tmp(3:end);
 
@@ -97,7 +97,7 @@ function out = readRegister(dev, reg, numbytes, precision)
   if (strcmp (precision,'uint16') || strcmp (precision,'int16'))
     sz = sz/2;
     out = typecast (out, precision);
-    if (endian == 'L')
+    if (endian != getEndian(dev.parent))
       out = swapbytes (out); 
     endif
   else
